@@ -1,7 +1,9 @@
+import datetime
 import re
+import uuid
 from collections import defaultdict
 from functools import lru_cache
-from textwrap import dedent, indent
+from textwrap import dedent
 from urllib.parse import urljoin
 
 import django.db.models.options as options
@@ -12,15 +14,11 @@ from django.contrib.postgres.indexes import GinIndex
 from django.contrib.postgres.search import SearchQuery, SearchRank, SearchVectorField
 from django.core.exceptions import ObjectDoesNotExist
 from django.db import connections, models
-from django.db.utils import NotSupportedError
 from django.template import Context, Template
 
 from urlid_graph import settings as urlid_graph_settings
 from urlid_graph.utils import random_name
 
-from . import formatting
-
-
 options.DEFAULT_NAMES = options.DEFAULT_NAMES + ("entity_name", "entity_uuid", "search_fields", "slug")
 
 
@@ -60,20 +58,7 @@ class EntityQuerySet(models.QuerySet):
         return self.filter(name__in=[name.lower().strip() for name in names])
 
 
-class BrasilIOEntityQuerySet(EntityQuerySet):
-    def get_queryset(self):
-        return (
-            super()
-            .get_queryset()
-            .filter(
-                base_url="https://id.brasil.io/",
-                version=1,
-            )
-        )
-
-
 class Entity(models.Model):
-    brasilio = BrasilIOEntityQuerySet.as_manager()
     objects = EntityQuerySet.as_manager()
 
     uuid = models.UUIDField(primary_key=True)
@@ -81,6 +66,10 @@ class Entity(models.Model):
     name = models.TextField(blank=False, null=False)
     version = models.TextField(blank=False, null=False)
 
+    class Meta:
+        verbose_name = "Entity"
+        verbose_name_plural = "Entities"
+
     @property
     def url(self):
         return urljoin(self.base_url, f"/{self.name}/v{self.version}/")
@@ -130,9 +119,7 @@ class ObjectMixin:
         if ObjectModel is None:
             raise ValueError("Model class not found fo entity UUID {}".format(self.entity.uuid))
 
-        return ObjectModel.objects.filter(object_uuid=self.uuid).order_by(
-            models.F("updated_at").asc(nulls_first=True)
-        )
+        return ObjectModel.objects.filter(object_uuid=self.uuid).order_by(models.F("updated_at").asc(nulls_first=True))
 
     @cached_property
     def properties(self):
@@ -158,7 +145,7 @@ class ObjectMixin:
                         "value": value,
                         "value_type": 1,  # TODO: change?
                         "source": "?",  # TODO: change?
-                        "value_datetime": obj.updated_at.isoformat() if obj.updated_at else None
+                        "value_datetime": obj.updated_at.isoformat() if obj.updated_at else None,
                     }
                 )
         return data
@@ -195,8 +182,6 @@ class PropertyQuerySet(models.QuerySet):
         return self.filter(object_id__in=[obj.uuid for obj in objs])
 
 
-
-
 class EntityConfigManager(models.Manager):
     def get_queryset(self, *args, **kwargs):
         return super().get_queryset(*args, **kwargs).filter(config_type=ElementConfig.ENTITY_CONFIG)
@@ -262,6 +247,8 @@ class ElementConfig(models.Model):
 
     class Meta:
         unique_together = ["config_type", "name", "parent_type", "parent_name"]
+        verbose_name = "Element configuration"
+        verbose_name_plural = "Element configurations"
 
 
 class EntityConfig(ElementConfig):
@@ -363,8 +350,7 @@ class ObjectRepositoryQuerySet(models.QuerySet):
         total_objects = row_count[1]
 
         string_agg = " || ' ' || ".join(
-            f"COALESCE(STRING_AGG({field_name}, ' '), '')"
-            for field_name in Model._meta.search_fields
+            f"COALESCE(STRING_AGG({field_name}, ' '), '')" for field_name in Model._meta.search_fields
         )
         insert_sql = dedent(
             f"""
@@ -427,14 +413,14 @@ class ObjectRepositoryQuerySet(models.QuerySet):
         return qs
 
     def search_many_entities(self, query, entities=all, limit_per_entity=10):
-        """Search BrasilIO objects in more than one entity, by name
+        """Search objects in more than one entity, by name
 
         Prefer to use this method since it also filter objects by entity (more performance)"""
 
         if entities is all:
-            entities = Entity.brasilio.all()
+            entities = Entity.objects.all()
         else:
-            entities = Entity.brasilio.by_name(entities)
+            entities = Entity.objects.by_name(entities)
 
         results = []
         for entity in entities:
@@ -448,14 +434,13 @@ class ObjectRepository(ObjectMixin, models.Model):
     objects = ObjectRepositoryQuerySet.as_manager()
 
     uuid = models.UUIDField(primary_key=True)
-    entity = models.ForeignKey(
-        Entity, on_delete=models.DO_NOTHING, db_constraint=False, db_index=False
-    )
+    entity = models.ForeignKey(Entity, on_delete=models.DO_NOTHING, db_constraint=False, db_index=False)
     search_data = SearchVectorField(null=True)
 
     class Meta:
         indexes = [GinIndex(fields=["search_data"])]
-        unique_together = ["entity", "uuid"]
+        verbose_name = "Object repository"
+        verbose_name_plural = "Object repositories"
 
 
 class SavedGraph(models.Model):
@@ -469,6 +454,8 @@ class SavedGraph(models.Model):
 
     class Meta:
         unique_together = (("user", "name"),)
+        verbose_name = "Saved graph"
+        verbose_name_plural = "Saved graphs"
 
 
 class ObjectModelMixin(models.Model):
@@ -476,11 +463,7 @@ class ObjectModelMixin(models.Model):
     updated_at = models.DateTimeField(null=True, blank=True)
 
     def raw_serialize(self):
-        return {
-            field.name: getattr(self, field.name)
-            for field in self._meta.fields
-            if field.name != "search_data"
-        }
+        return {field.name: getattr(self, field.name) for field in self._meta.fields if field.name != "search_data"}
 
     class Meta:
         abstract = True
@@ -502,6 +485,7 @@ class DatasetModel(models.Model):
     @classmethod
     def fields(cls):
         from django.db import connection
+
         return [
             {
                 "name": field.name,
@@ -509,13 +493,14 @@ class DatasetModel(models.Model):
                 "type": field.db_type(connection),
                 "description": field.help_text,
             }
-            for field in cls._meta.fields if field.name not in ("id", "object_uuid")
+            for field in cls._meta.fields
+            if field.name not in ("id", "object_uuid")
         ]
 
     @cached_property
     def serializers(self):
         return {
-            attribute[len("serialize_"):]: getattr(self, attribute)
+            attribute[len("serialize_") :]: getattr(self, attribute)
             for attribute in dir(self)
             if attribute.startswith("serialize_")
         }
@@ -543,3 +528,73 @@ class DatasetModel(models.Model):
 
     class Meta:
         abstract = True
+
+
+class JobLog(models.Model):
+    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
+    name = models.CharField(max_length=255)
+    description = models.CharField(max_length=255)
+    created_at = models.DateTimeField(auto_now_add=True)
+
+    class Meta:
+        ordering = ("-created_at",)
+
+    @property
+    def current_step(self):
+        return self.steps.first().step
+
+    @property
+    def progress(self):
+        step = self.steps.first()
+        if step.done is not None and step.total is not None and step.total != 0:
+            return f"{100 * step.done / step.total:5.2f}%"
+        else:
+            result = step.step + (f" ({step.action})" if step.action else "")
+            if self.steps.filter(action="error").count() > 0:
+                result += " - has error"
+            return result
+
+    @property
+    def eta(self):
+        step = self.steps.first()
+        if step.done is None or step.total is None or step.total == 0:
+            return "-"
+        first_step = self.steps.exclude(pk=step.pk).filter(step=step.step, done__gt=0).order_by("created_at").first()
+        if first_step is None:
+            return "-"
+        to_finish = step.total - step.done
+        delta_done = step.done - first_step.done
+        delta_time = (step.created_at - first_step.created_at).total_seconds()
+        speed = delta_done / delta_time
+        result = str(datetime.timedelta(seconds=to_finish / speed))
+        return result.split(".")[0]
+
+    @property
+    def last_updated_at(self):
+        return self.steps.first().created_at
+
+    def __str__(self):
+        return f"{self.name} - {self.description}: {self.id}"
+
+    def save(self, *args, **kwargs):
+        if self._state.adding:
+            super().save(*args, **kwargs)
+            LogStep.objects.create(job=self, action="started", step="created")
+        else:
+            super().save(*args, **kwargs)
+
+
+class LogStep(models.Model):
+    job = models.ForeignKey(JobLog, related_name="steps", on_delete=models.CASCADE)
+    action = models.CharField(max_length=255)
+    step = models.CharField(max_length=255)
+    done = models.FloatField(null=True, blank=True)
+    total = models.FloatField(null=True, blank=True)
+    message = models.TextField(null=True, blank=True)
+    created_at = models.DateTimeField(auto_now_add=True)
+
+    class Meta:
+        ordering = ("-created_at",)
+
+    def __str__(self):
+        return f"{self.step} - {self.action}: {self.job.id}"
