@@ -1,8 +1,7 @@
 import agensgraph  # noqa
 from django.conf import settings
 
-
-USER_SETTINGS = getattr(settings, 'URLID_GRAPH', dict())
+USER_SETTINGS = getattr(settings, "URLID_GRAPH", dict())
 
 DEFAULTS = {
     "SEARCH_LANGUAGE": "pg_catalog.portuguese",
